/**********************************************************************
Copyright (c) 2010 Alexander Kerner. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 ***********************************************************************/

package net.sf.mvc.prototype.example.swing;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.swing.SwingUtilities;

import net.sf.mvc.prototype.MVC;
import net.sf.mvc.prototype.controller.AbstractInputOutputController;

/**
 * 
 * 
 * A custom controller. This class serves as both input controller and output
 * controller.
 * 
 * 
 * @author <a href="mailto:alex.kerner.24@googlemail.com">Alexander Kerner</a>
 * @version 2010-10-31
 * @see net.sf.mvc.prototype.controller.InputController InputController
 * @see net.sf.mvc.prototype.controller.OutputController OutputController
 * @see net.sf.mvc.prototype.controller.InputOutputController InputOutputController
 * 
 */
public class MyController extends AbstractInputOutputController<MyModel, MyView> {

	/**
	 * Drop all events coming in to other thread in oder to escape from the
	 * AWT-Event thread.
	 */
	private final ExecutorService exe = Executors.newSingleThreadExecutor();

	public static void main(String[] args) {

		// create controller
		final MyController controller = new MyController();

		// create model
		final MyModel model = new MyModel(controller);

		// create view
		final MyView view = new MyView();

		// use MVC helper method to init MVC application properly
		MVC.init(model, view, view, controller, controller);

		// start application
		// use AWT-Event-Dispatching-_Thread
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				view.showView();
			}
		});
	}

	public void shutdownRequest() {

		// escape from AWT-event thread
		exe.submit(new Runnable() {
			public void run() {
				System.out.println("View requested application shutdown");
				shutdown();
			}
		});
	}

	public void infoMsg(final String msg) {
		// this must be done by AWT-event thread
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				for (MyView v : views) {
					v.infoMsg(msg);
				}
			}
		});
	}

	public void warnMsg(final String msg) {
		// this must be done by AWT-event thread
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				for (MyView v : views) {
					v.warnMsg(msg);
				}
			}
		});
	}

	public void errorMsg(final String msg) {
		// this must be done by AWT-event thread
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				for (MyView v : views) {
					v.errorMsg(msg);
				}
			}
		});
	}

	public void shutdown() {
		// this must be done by AWT-event thread
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				System.out.println("Bringing down view");
				for (MyView v : views) {
					v.destroyView();
				}
			}
		});

		System.out.println("Bringing down model");
		for (MyModel m : models) {
			m.shutdown();
		}
		
		System.out.println("cleaining up");
		exe.shutdown();
	}
}
