/**********************************************************************
Copyright (c) 2010 Alexander Kerner. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 ***********************************************************************/

package net.sf.mvc.prototype.model;

import net.sf.mvc.prototype.controller.InputController;
import net.sf.mvc.prototype.controller.OutputController;
import net.sf.mvc.prototype.view.OutputView;

/**
 * 
 * 
 * Model of the MVC-Pattern. A {@code Model} listens to events from
 * {@link InputController} and creates events send to one or more
 * {@link OutputController}.
 * 
 * @author <a href="mailto:alex.kerner.24@googlemail.com">Alexander Kerner</a>
 * @version 2010-10-31
 * @param D
 *            {@link OutputController} handling events coming from this
 *            {@code Model}
 * @see InputController
 * @see OutputController
 */
public interface Model<D extends OutputController<? extends OutputView>> {

	/**
	 * 
	 * Add an {@link OutputController} to this {@code Model}. All events created by this
	 * {@code Model} will be broadcasted to all registered
	 * {@link OutputController}s.
	 * 
	 * @param controller
	 *            {@link OutputController} that is added to this {@code Model}
	 */
	void addControllerOut(D controller);

	/**
	 * 
	 * Remove an {@link OutputController} from this {@code Model}. Created events will
	 * not anymore send to this {@link OutputController}.
	 * 
	 * @param controller
	 *            {@link OutputController} that is removed from this {@code Model}
	 */
	void removeControllerOut(D controller);

	/**
	 * Shutdown this {@code Model}. A {@code Model} that has been shutdown does
	 * not process anymore events that are send to this {@code Model}, nor does
	 * a shutdown {@code Model} send anymore events to registered
	 * {@link OutputController}s.
	 * <p>
	 * Shutting down a {@code Model} will also shutdown all this {@code Model}'s
	 * {@link java.util.concurrent.ExecutorService ExecutorService}ses.
	 * </p>
	 * 
	 */
	void shutdown();

}
